package jsfSupport.converters;


import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.validator.ValidatorException;

import jsfSupport.managedBeans.ConverterFactory;
import persistence.jpa.entities.CompanyEntity;

public abstract class EntityConverter implements Converter {

	private static final String NULL = "null";
	private ConverterFactory  convFactory;
	
	@Override
	public Object getAsObject(FacesContext context, UIComponent component, String value) {
		if(NULL.equals(value)){
			return null;
		}
		try {
			Long id = Long.parseLong(value);
			return getObject(getConvFactory(context), id);
		} catch (NumberFormatException e) {
			FacesMessage msg = new FacesMessage("Value '" + value + "' is not long number.", 
					"Value '" + value + "' is not object id (long type).");
			msg.setSeverity(FacesMessage.SEVERITY_FATAL);
			throw new ValidatorException(msg);
		}

	}

	@Override
	public String getAsString(FacesContext context, UIComponent component, Object value) {
		if(value == null){
			return NULL;
		}
		if(value instanceof CompanyEntity){
			return ((CompanyEntity)value).getId().toString();
		} else {
			FacesMessage msg = new FacesMessage("Object is not instance of " + CompanyEntity.class.getName() + ".", 
					"Object is not instance of " + CompanyEntity.class.getName() + ".");
			msg.setSeverity(FacesMessage.SEVERITY_FATAL);
			throw new ValidatorException(msg);
			
		}
	}
	
	public ConverterFactory getConvFactory(FacesContext context) {
		if(convFactory == null){
			ELContext el = context.getELContext();
			ExpressionFactory factory = context.getApplication().getExpressionFactory(); 
			convFactory = (ConverterFactory)factory.createValueExpression(el, "#{converterFactory}", ConverterFactory.class).getValue(el);
		}
		return convFactory;
	}

	protected abstract Object getObject(ConverterFactory factory, Long id);
}
