package persistence.jpa.dao;

import java.util.List;

import javax.persistence.EntityManager;

import persistence.jpa.entities.CompanyEntity;

public abstract class EntityDao<T extends CompanyEntity> {

    protected EntityManager em;

    public EntityDao(EntityManager em){
        this.em = em;
    }
    public T savePosition(T e){
    	if(e.getId() == null){
    		em.persist(e);
    	} else {
    		em.merge(e);
    	}
        em.flush();
        return e;
    }

    public void delete(T e){
    	em.merge(e);
    	em.remove(e);
    	em.flush();
    }
    
    public T find(Long id){
    	return (T)em.find(getEntityClass(), id);
    }
    
    public List<T> getAll(int startIndex, int length){
        List<T> col = em.createNamedQuery(getEntityClass().getSimpleName() + ".all").setFirstResult(startIndex).setMaxResults(length).getResultList();
        return col;
    }
    
    public int getAllCount(){
        return (Integer)em.createNamedQuery(getEntityClass().getSimpleName() + ".allCount").getSingleResult();
    }


    protected abstract Class getEntityClass();
}
